package com.ycwl.basic.task;

import cn.hutool.core.date.DateUtil;
import com.ycwl.basic.mapper.FaceMapper;
import com.ycwl.basic.mapper.FaceSampleMapper;
import com.ycwl.basic.mapper.ScenicMapper;
import com.ycwl.basic.mapper.SourceMapper;
import com.ycwl.basic.mapper.VideoMapper;
import com.ycwl.basic.model.pc.face.req.FaceReqQuery;
import com.ycwl.basic.model.pc.face.resp.FaceRespVO;
import com.ycwl.basic.model.pc.faceSample.req.FaceSampleReqQuery;
import com.ycwl.basic.model.pc.faceSample.resp.FaceSampleRespVO;
import com.ycwl.basic.model.pc.scenic.entity.ScenicConfigEntity;
import com.ycwl.basic.model.pc.scenic.req.ScenicReqQuery;
import com.ycwl.basic.model.pc.scenic.resp.ScenicRespVO;
import com.ycwl.basic.model.pc.source.req.SourceReqQuery;
import com.ycwl.basic.model.pc.source.resp.SourceRespVO;
import com.ycwl.basic.model.pc.video.req.VideoReqQuery;
import com.ycwl.basic.model.pc.video.resp.VideoRespVO;
import com.ycwl.basic.repository.ScenicRepository;
import com.ycwl.basic.service.task.TaskFaceService;
import com.ycwl.basic.storage.StorageFactory;
import com.ycwl.basic.storage.adapters.IStorageAdapter;
import com.ycwl.basic.storage.entity.StorageFileObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Objects;

import static com.ycwl.basic.constant.FaceConstant.USER_FACE_DB_NAME;

@Component
@EnableScheduling
@Slf4j
@Profile("prod")
public class FaceCleaner {
    @Autowired
    private ScenicMapper scenicMapper;
    @Autowired
    private TaskFaceService faceService;
    @Autowired
    private FaceSampleMapper faceSampleMapper;
    @Autowired
    private SourceMapper sourceMapper;
    @Autowired
    private VideoMapper videoMapper;
    @Autowired
    private ScenicRepository scenicRepository;
    @Autowired
    private FaceMapper faceMapper;

    @Scheduled(cron = "0 0 4 * * ?")
    public void clean(){
        ScenicReqQuery scenicQuery = new ScenicReqQuery();
        List<ScenicRespVO> scenicList = scenicMapper.list(scenicQuery);
        scenicList.forEach(scenic -> {
            log.info("当前景区{}，开始删除人脸样本", scenic.getName());
            faceService.batchDeleteExpiredFace(scenic.getId());
        });
    }

    @Scheduled(cron = "0 0 3 * * ?")
    public void deleteExpireFace() {
        ScenicReqQuery scenicQuery = new ScenicReqQuery();
        List<ScenicRespVO> scenicList = scenicMapper.list(scenicQuery);
        scenicList.parallelStream().forEach(scenic -> {
            ScenicConfigEntity scenicConfig = scenicRepository.getScenicConfig(scenic.getId());
            Integer sampleStoreDay = scenicConfig.getSampleStoreDay();
            if (sampleStoreDay == null) {
                log.info("当前景区{}，人脸样本保存天数未设置，默认3天", scenic.getName());
                sampleStoreDay = 3;
            }
            FaceReqQuery req = new FaceReqQuery();
            req.setScenicId(scenic.getId());
            req.setUpdateEndTime(DateUtil.offsetDay(DateUtil.beginOfDay(new Date()), -sampleStoreDay));
            List<FaceRespVO> list = faceMapper.list(req);
            list.forEach(face -> {
                boolean result = faceService.deleteFaceSample(USER_FACE_DB_NAME+face.getScenicId(), face.getId().toString());
                if (result) {
                    faceMapper.deleteById(face.getId());
                }
            });
            log.info("当前景区{}，删除人脸样本{}个", scenic.getName(), list.size());
        });
    }

    @Scheduled(cron = "0 0 3 * * ?")
    public void deleteNotBuySource(){
        ScenicReqQuery scenicQuery = new ScenicReqQuery();
        List<ScenicRespVO> scenicList = scenicMapper.list(scenicQuery);
        scenicList.parallelStream().forEach(scenic -> {
            ScenicConfigEntity scenicConfig = scenicRepository.getScenicConfig(scenic.getId());
            if (scenicConfig == null) {
                log.info("当前景区{}，无配置信息", scenic.getName());
                return;
            }
            if (scenicConfig.getUserSourceExpireDay() == null) {
                log.info("当前景区{}，人脸样本过期天数未设置", scenic.getName());
                return;
            }
            int expireDay = scenicConfig.getUserSourceExpireDay();
            Date endDate = DateUtil.offsetDay(DateUtil.beginOfDay(new Date()), -expireDay);
            int deleteCount = sourceMapper.deleteNotBuyRelations(scenic.getId(), endDate);
            log.info("当前景区{}，删除关联素材{}个", scenic.getName(), deleteCount);
        });
    }

    @Scheduled(cron = "0 0 3 * * ?")
    public void deleteNotBuyVideos(){
        ScenicReqQuery scenicQuery = new ScenicReqQuery();
        List<ScenicRespVO> scenicList = scenicMapper.list(scenicQuery);
        scenicList.parallelStream().forEach(scenic -> {
            ScenicConfigEntity scenicConfig = scenicRepository.getScenicConfig(scenic.getId());
            if (scenicConfig == null) {
                log.info("当前景区{}，无配置信息", scenic.getName());
                return;
            }
            if (scenicConfig.getVideoStoreDay() == null) {
                log.info("当前景区{}，VLOG过期天数未设置", scenic.getName());
                return;
            }
            int expireDay = scenicConfig.getVideoStoreDay();
            Date endDate = DateUtil.offsetDay(DateUtil.beginOfDay(new Date()), -expireDay);
            int deleteCount = videoMapper.deleteNotBuyRelations(scenic.getId(), endDate);
            log.info("当前景区{}，删除VLOG{}个", scenic.getName(), deleteCount);
        });
    }

    @Scheduled(cron = "0 30 3 * * ?")
    public void deleteExpiredSource(){
        ScenicReqQuery scenicQuery = new ScenicReqQuery();
        List<ScenicRespVO> scenicList = scenicMapper.list(scenicQuery);
        scenicList.parallelStream().forEach(scenic -> {
            ScenicConfigEntity scenicConfig = scenicRepository.getScenicConfig(scenic.getId());
            if (scenicConfig == null) {
                log.info("当前景区{}，无配置信息", scenic.getName());
                return;
            }
            int imageSourceExpireDay = 7;
            int videoSourceExpireDay = 7;
            if (scenicConfig.getImageSourceStoreDay() != null) {
                imageSourceExpireDay = scenicConfig.getImageSourceStoreDay();
            } else {
                log.info("当前景区{}，原始素材保存天数未设置，默认7天", scenic.getName());
            }
            if (scenicConfig.getVideoSourceStoreDay() != null) {
                videoSourceExpireDay = scenicConfig.getVideoSourceStoreDay();
            } else {
                log.info("当前景区{}，原始素材保存天数未设置，默认7天", scenic.getName());
            }
            if (Integer.valueOf(1).equals(scenicConfig.getDisableSourceVideo())) {
                return;
            }
            if (Integer.valueOf(1).equals(scenicConfig.getDisableSourceImage())) {
                return;
            }
            log.info("当前景区{}，开始删除原始素材", scenic.getName());
            Date endDate = DateUtil.offsetDay(DateUtil.beginOfDay(new Date()), -videoSourceExpireDay);
            int deleteVideoSourceCount = sourceMapper.deleteNotRelateSource(1, endDate);
            log.info("当前景区{}，删除原始视频素材{}个", scenic.getName(), deleteVideoSourceCount);
            log.info("当前景区{}，开始删除原始图片素材", scenic.getName());
            Date endDate2 = DateUtil.offsetDay(DateUtil.beginOfDay(new Date()), -imageSourceExpireDay);
            int deleteImageSourceCount = sourceMapper.deleteNotRelateSource(2, endDate2);
            log.info("当前景区{}，删除原始图片素材{}个", scenic.getName(), deleteImageSourceCount);
        });
    }

    @Scheduled(cron = "0 0 5 * * ?")
    public void clearOss(){
        cleanFaceSampleOss();
        cleanSourceOss();
        cleanVideoOss();
    }
    private void cleanFaceSampleOss() {
        log.info("开始清理人脸文件");
        List<FaceSampleRespVO> faceSampleRespVOS = faceSampleMapper.list(new FaceSampleReqQuery());
        IStorageAdapter adapter = StorageFactory.use("faces");
        List<StorageFileObject> fileObjectList = adapter.listDir("user-face");
        fileObjectList.parallelStream().forEach(fileObject -> {
            if(faceSampleRespVOS.parallelStream().noneMatch(faceSampleRespVO -> faceSampleRespVO.getFaceUrl().contains(fileObject.getFullPath()))){
                log.info("删除人脸文件：{}", fileObject);
                adapter.deleteFile(fileObject.getFullPath());
            }
        });
    }
    private void cleanSourceOss() {
        log.info("开始清理源视频素材文件");
        List<SourceRespVO> list = sourceMapper.list(new SourceReqQuery());
        IStorageAdapter adapter = StorageFactory.use("assets-ext");
        List<StorageFileObject> fileObjectList = adapter.listDir("video-source");
        fileObjectList.parallelStream().forEach(fileObject -> {
            if (list.parallelStream().filter(videoRespVO -> Objects.nonNull(videoRespVO.getVideoUrl())).noneMatch(videoRespVO -> videoRespVO.getVideoUrl().contains(fileObject.getFullPath()))){
                log.info("删除源视频素材文件：{}", fileObject);
                adapter.deleteFile(fileObject.getFullPath());
            } else {
                log.info("源视频素材文件存在关系：{}，未删除", fileObject);
            }
        });
        log.info("开始清理源图片素材文件");
        IStorageAdapter imageAdapter = StorageFactory.use("faces");
        List<StorageFileObject> imageFileObjectList = imageAdapter.listDir("user-photo");
        imageFileObjectList.parallelStream().forEach(fileObject -> {
            if (list.parallelStream().filter(videoRespVO -> Objects.nonNull(videoRespVO.getUrl())).noneMatch(videoRespVO -> videoRespVO.getUrl().contains(fileObject.getFullPath()))){
                log.info("删除图片文件：{}", fileObject);
                imageAdapter.deleteFile(fileObject.getFullPath());
            } else {
                log.info("图片文件存在关系：{}，未删除", fileObject);
            }
        });
    }
    private void cleanVideoOss() {
        log.info("开始清理视频文件");
        List<VideoRespVO> videoRespVOS = videoMapper.list(new VideoReqQuery());
        IStorageAdapter adapter = StorageFactory.use("video");
        List<StorageFileObject> fileObjectList = adapter.listDir("");
        fileObjectList.parallelStream().forEach(fileObject -> {
            if (videoRespVOS.parallelStream().noneMatch(videoRespVO -> videoRespVO.getVideoUrl().contains(fileObject.getFullPath()))){
                log.info("删除视频文件：{}", fileObject);
                adapter.deleteFile(fileObject.getFullPath());
            } else {
                log.info("视频文件存在关系：{}，未删除", fileObject);
            }
        });
    }
}
